--=======================================================================================================
--  BALESEE HELPER FUNCTIONS
--
-- Purpose:		Allows bales and pallets to show up on the PDA map as hotspots.
-- Author:		Mmtrx		
-- Changelog:
--  v1.0		03.01.2019	original FS17 version by akuenzi (akuenzi@gmail.com)
--	v1.1.0		28.08.2019	updates for FS19, added user interface  
--  v1.1.1		17.09.2019  added pallette support.
--  v1.1.2		08.10.2019	save statistics / add legend in debug mode 
--=======================================================================================================
-- ---------------Helper functions-----------------------------------------------------------
function BaleSee:getColor(objectId)
	-- return color depending on filltype of bale object 
	local ft = objectId.fillType 		-- works, if it's a bale
	if ft == nil then 					-- it's a pallet, vehicle object.
		ft = objectId:getFillUnitFillType(1)
	end
	return Utils.getNoNil(BaleSee.colors[ft], {1,1,1,1}) 		-- default
end;
function BaleSee:getIcon(objectId)
	-- Hotspot bale icons that will show on the map and in the Escape menu.
	local files =	BaleSee.icons
	-- return icon image depending on filltype of bale object 
	local image   = Utils.getFilename(files.squareStraw, BaleSee.modDir); -- default image: FillType.STRAW
	local fill 	  = objectId.fillType
	
	-- select correct image, based on filltype
	if objectId.baleDiameter then
		image = Utils.getFilename(files.roundStraw, BaleSee.modDir);
		if     fill == FillType.DRYGRASS_WINDROW then
			image = Utils.getFilename(files.roundHay, BaleSee.modDir);
		elseif fill == FillType.GRASS_WINDROW then
			image = Utils.getFilename(files.roundGrass, BaleSee.modDir);
		elseif fill == FillType.SILAGE then
			image = Utils.getFilename(files.roundSilage, BaleSee.modDir);
		end;			
	else
		if     fill == FillType.DRYGRASS_WINDROW then
			image = Utils.getFilename(files.squareHay, BaleSee.modDir);
		elseif fill == FillType.GRASS_WINDROW then
			image = Utils.getFilename(files.squareGrass, BaleSee.modDir);
		elseif fill == FillType.SILAGE then
			image = Utils.getFilename(files.squareSilage, BaleSee.modDir);
		end;
	end;	
	return image
end;
function BaleSee:getImage(self)
	-- find image source for the pallet icon display
	local image = 		Utils.getFilename(BaleSee.icons.otherPallet, BaleSee.modDir); --default image
	local fillType = 	self:getFillUnitFillType(1)
	if fillType == FillType.WOOL then
			image = Utils.getFilename(BaleSee.icons.woolPallet, BaleSee.modDir);
	elseif self.configFileName then
		local storeItem = g_currentMission.shopController.storeManager.xmlFilenameToItem[
			string.lower(self.configFileName)]
		if storeItem and storeItem.imageFilename then
			image = storeItem.imageFilename
	-- error in FS19 data: fillablePallet.xml, woolPallet.xml <image> says: data/store/store_pallet_saplings.png
	-- chgd to $data/objects/pallets/palletPoplar/store_pallet_saplingsPoplar.png
	-- (the png is the std plain wooden box pallet)
			if string.match(image, "data/store/") then
				image = "data/objects/pallets/palletPoplar/store_pallet_saplingsPoplar.png"
			end
		end;
	end;
	return image
end
function BaleSee:delhot(hotspot)
	if hotspot ~= nil then
		g_currentMission.hud.ingameMap:removeMapHotspot(hotspot);
		for i = 1, table.getn(BaleSee.bHotspots) do
			if BaleSee.bHotspots[i][1] == hotspot then
		    	table.remove(BaleSee.bHotspots, i)
		    	break
			end
		end
	end;
end;
function BaleSee:makeLegend()
	-- show all bale/pallet hotspot markers:
	local w,h = getNormalizedScreenValues(16,16)
	local x,z = -1000, -1000
	local uvP = getNormalizedUVs({520, 520, 240, 240})
	local uvB = getNormalizedUVs({776, 776, 240, 240})
	local function isbale( ft )		-- true if ft is bale fillType
		local ftbale = {FillType.STRAW, FillType.GRASS_WINDROW, FillType.DRYGRASS_WINDROW,
			FillType.SILAGE	}
		for _,f in ipairs(ftbale) do
			if f == ft then return true end
		end
		return false
	end;

	for i=1,52 do
		if BaleSee.colors[i] ~= nil then 
			local color = BaleSee.colors[i]
			local hotspot = MapHotspot:new(string.format("%d %s",i,BaleSee.ft[i].title), 
				MapHotspot.CATEGORY_TOUR)
			hotspot:setIconScale(0.8)
			hotspot:setSize(w,h)	-- size for small cicrcles
			hotspot.verticalAlignment = Overlay.ALIGN_VERTICAL_MIDDLE
			hotspot:setImage(nil, uvP, color)
			hotspot:setBackgroundImage(nil, uvP, nil)
			if isbale(i) then
				hotspot:setSize(0.9*w,0.9*h)
				hotspot:setImage(nil, uvB, color)
				hotspot:setBackgroundImage(nil, uvB, nil)
			end			
			hotspot:setWorldPosition(x, z)			-- also sets the x,z MapPos
	--		hotspot:setTextOptions(textSize, textBold, textOffsetY, textColor, 
	--			verticalAlignment, textAlignment)
			hotspot:setTextOptions(0.5*h, nil, nil, nil, nil, nil)
			hotspot:setText(hotspot.name,false,true)
			hotspot.enabled = false
			g_currentMission.hud.ingameMap:addMapHotspot(hotspot) 
			table.insert(BaleSee.legend, {hotspot,color}) 
			z = z +70	
		end				
	end;
end;
function BaleSee:printStats()
	local h,o,oi,r,f 
	local ba = {
		[FillType.STRAW] = 				{0,0}, 
		[FillType.GRASS_WINDROW] = 		{0,0},
		[FillType.DRYGRASS_WINDROW] = 	{0,0},
		[FillType.SILAGE] = 			{0,0},
	}
	local bar, baq = 0, 0
	local pa = {
		[FillType.UNKNOWN] = 0,	
		[FillType.PIGFOOD] = 0,	
		[FillType.SEEDS] = 0,		
		[FillType.FERTILIZER] = 0, 
		[FillType.LIME] = 0,		
		[FillType.WHEAT] = 0,		
		[FillType.OAT] = 0,		
		-- pallet tanks:
		[FillType.LIQUIDFERTILIZER] = 0,
		[FillType.HERBICIDE] = 0,	
		-- other pallets:
		[FillType.TREESAPLINGS]= 0,
		[FillType.SUGARCANE] = 0,	
		[FillType.SUGARBEET] = 0,	
		[FillType.WOOL] = 0,		
		[FillType.EGG] = 0,		
		[FillType.MILK] = 0,		
		[FillType.POTATO] = 0,		
		[FillType.POPLAR] = 0					
	}
	for i=1, #BaleSee.bHotspots do
		h = BaleSee.bHotspots[i][1]
		oi = h.objectId
		r = Utils.getNoNil(getUserAttribute(oi, "isRoundbale"), false)
		o = g_currentMission:getNodeObject(oi)
		f = o.fillType
		if r then 	ba[f][1] = ba[f][1] +1; bar = bar+1
		else 		ba[f][2] = ba[f][2] +1; baq = baq+1
		end
		if BaleSee.debug then
			print(string.format("** %2d: %s %s %d at %4.0f %4.0f", i, BaleSee.isRound[r],
			h.fullViewName, oi, h.xMapPos,h.zMapPos ))
		end
	end
	for i=1, #BaleSee.pHotspots do
		h = BaleSee.pHotspots[i][1]
		oi = h.objectId
		o = g_currentMission:getNodeObject(oi)
		if BaleSee.debug then
			print(string.format("** %2d: %s %d at %4.0f %4.0f",i,h.fullViewName,oi, 
			h.xMapPos,h.zMapPos))
		end
		f = Utils.getNoNil(o:getFillUnitFillType(1),FillType.UNKNOWN)
		--print(string.format("** fillType %2d. object %s",f,o)) 
		if pa[f] ~= nil then pa[f] = pa[f] +1 
		else 
			print(string.format("**BaleSee error: unknown pallet filltype %d in object:",
				f))
			DebugUtil.printTableRecursively(o,".",0,0);
		end
	end
	local sep = "--------------------------\n"
	local myfile = getUserProfileAppPath().."BaleSee.txt"
	local mf = io.open(myfile, "w")
	if mf then
		mf:write(sep,"Ballen    rund  quad\n",
			string.format("         %5d %5d\n",bar,baq), sep)
		for t,c in pairs(ba) do
			mf:write(string.format("%8s %5d %5d\n",BaleSee.ft[t].title, c[1], c[2]))
		end
		mf:write(sep,string.format("Paletten %17d\n", #BaleSee.pHotspots),sep)
		local txt,utl,pre 
		for t,c in pairs(pa) do
			if c > 0 then
				txt = BaleSee.ft[t].title
				pre = ""
				utl = string.len(txt) - utf8Strlen(txt)
				if utl > 0 then
					pre = string.rep(" ",utl);end
				mf:write(pre..string.format("%20s %5d\n", txt, c))
			end
		end
		mf:write(sep)
		mf:close()
	else
		print("**BaleSee error: could not open file "..myfile)
	end
end
function BaleSee:toggleVis(spots, on)
	-- show / hide some map hotspots
	for i = 1, #spots do
		spots[i][1].enabled = on
		spots[i][1].hasDetails = on
	end
end;
function BaleSee:togglePalCol()
	-- toggle BaleSee.colPalActivate
	BaleSee.colPalActivate = not BaleSee.colPalActivate
	if BaleSee.debug then
		print("--BaleSee: set colPalActivate to " .. tostring(BaleSee.colPalActivate));
	end
	if BaleSee.colPalActivate then
	-- for each pallet hotspot: reset it to uvP marker
		for _, h in pairs(BaleSee.pHotspots) do
			-- h is {hotspot, color, image}
			hs = h[1]
			--remove image, bgImage, set uv and color
			hs:setImage 		 (nil, BaleSee.uvP, h[2])
			hs:setBackgroundImage(nil, BaleSee.uvP, nil)	
			hs:setIconScale		 (0.7)
			hs:setSize			 (BaleSee.spotWid, BaleSee.spotHei)	
		end
	else    -- reset hotspots to Icons:
		for _, h in pairs(BaleSee.pHotspots) do
			hs = h[1]
			--remove uv, col. Set image, bgImage 
			hs:setImage 		 (h[3], nil, nil)
			hs:setBackgroundImage(BaleSee.bgImage, nil, nil)	
			hs:setIconScale 	 (1)
			hs:setSize			 (BaleSee.iconWid, BaleSee.iconHei)	
		end
	end
end;

